//  LUNCONV.C
//  Functions for Goddess lunar calendar date conversion
//  Author: Peter Meyer
//  Last mod.: 1996-08-21

#include <STDIO.H>
#include <STDLIB.H>
#include <STRING.H>

#include "DATECONV.H"
#include "LUNCONV.H"

static long glc_base_jdn = 237819L;   //  -4061-01-10
static int leap_month = 24;           //  Xochiquetzal

static int month_length[26] =
{ 0, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30,
     29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30 };

/*--------------------*/
int leap_year(long year)
{
return ( !(year%4) || !(year%51) );
}

/*--------------------------*/
int glc_year_length(long year)
{
return ( 738 + leap_year(year) );
}

/*-----------------------------*/
void set_month_lengths(long year)
{
int m;

for ( m=1; m<=25; m++ )
    month_length[m] = 29 + (m%2);

if ( leap_year(year) )
    month_length[leap_month]++;
}

/*------------------------------*/
int lunar_month_length(int month)
{
if ( month < 1 || month > 25 )
    return ( 0 );
else
    return ( month_length[month] );
}

//  Checks Goddess lunar date for validity.
//  Sets "valid" flag to FALSE if date invalid, TRUE otherwise.
/*----------------------------*/
int goddess_date_valid(Date *dt)
{
long year = dt->year;
int month = dt->month;
int day = dt->day;

dt->valid = FALSE;

set_month_lengths(year);

if ( month >= 1 && month <= 25 )
    {
    if ( day >=1 && day <= month_length[month] )
        dt->valid = TRUE;
    }

return ( dt->valid );
}

//  Converts Goddess lunar date to Julian day number.
//  Sets "valid" flag to FALSE if date invalid, otherwise
//  returns the Julian day number in dt.jdn
//  and the Goddess cycle number (0-150605).
/*------------------------------*/
long goddess_date_to_jdn(Date *dt)
{
long year=dt->year;
int month=dt->month;
int day=dt->day;
int k;
long i, yr, n, m;

if ( !goddess_date_valid(dt) )
    return (-1L );

if ( year > 203  )
    {
    i = year/204;
    year -= i*204;
    m = i*150606L;
    }
else if ( year < 0 )
    {
    i = 1 - ((year+1)/204);
    year += i*204;
    m = i*(-150606L);
    }
else
    m = 0;

//  year is now in the range 0 through 203.
//  Add m to n below to get jdn.

//  Add days for years.

n = 0;
for ( yr=0; yr<year; yr++ )
    n += glc_year_length(yr);


//  Add days for months.

set_month_lengths(year);

for ( k=1; k<month; k++ )
    n += month_length[k];

//  Add days for days.

n += day - 1;

dt->jdn = n + m + glc_base_jdn;

return ( n );
}

//  Converts Julian day number to Goddess lunar date.
//  Returns Goddess cycle number (0-150605).
/*------------------------------*/
long jdn_to_goddess_date(Date *dt)
{
int month, year_length;
long gcn, year, y204=0;
long n = dt->jdn - glc_base_jdn;

if ( n > 150605 )
    {
    y204 = n/150606L;
    n -= y204*150606L;
    }
else if ( n < 0 )
    {
    y204 = 1 - ((n+1)/150606L);
    n += y204*150606L;
    y204 *= -1;
    }

gcn = n;

//  n is now in the range 0 through 150605.
//  Now get the year (0 through 203) and the month and day
//  and add 204*y204 to the year to get the target year.

//  Get year

year = 0;

while ( TRUE )
    {
    year_length = glc_year_length(year);
    if ( n < year_length )
        break;
    n -= year_length;
    year++;
    }

//  Get month

set_month_lengths(year);

month = 1;

do  {
    if ( n < month_length[month] )
        break;
    n -= month_length[month++];
    } while ( month < 25 );

dt->year = year + 204*y204;
dt->month = month;
dt->day = (int)n+1;
dt->valid = TRUE;

return ( gcn );
}
